@@ -1,4 +1,4 @@
-"""Springer Textbook Bulk Downloader
+"""Springer Textbook Bulk Download Tool
 """
 
 import typer
@@ -12,7 +12,7 @@ from .catalog import Catalog
 
 cli = typer.Typer()
 
-DOWNLOAD_REPORT = "DOWNLOAD_ERRORS.txt"
+DOWNLOAD_REPORT = "DOWNLOAD_REPORT.txt"
 
 
 @cli.callback()
@@ -35,13 +35,19 @@ def main(
         help="Choose a catalog topic.",
     ),
 ):
-    """Springer Textbook Bulk Download Tool
+    """
+
+    ![Downloading](https://github.com/JnyJny/springer_downloader/raw/master/demo/b.gif)
+
+    ![Longer Demo](https://github.com/JnyJny/springer_downloader/raw/master/demo/demo1_fast.gif)
+    __Springer Textbook Bulk Download Tool__
     
     **NOTICE**:
 
-    Author not affiliated with Springer and this tool is not authorized
-    or supported by Springer. Thank you to Springer for making these
-    high quality textbooks available at no cost. 
+    The author of this software is not affiliated with Springer and this
+    tool is not authorized or supported by Springer. Thank you to
+    Springer for making these high quality textbooks available at no
+    cost.
 
     \b
     >"With the Coronavirus outbreak having an unprecedented impact on
@@ -60,34 +66,32 @@ def main(
 
     [Source](https://www.springernature.com/gp/librarians/news-events/all-news-articles/industry-news-initiatives/free-access-to-textbooks-for-institutions-affected-by-coronaviru/17855960)
     
-    This tool automates the tasks of downloading the Excel-formatted
-    catalogs and downloading the files described in the catalog.
+    This tool automates the tasks of downloading the Springer provided
+    Excel-formatted catalogs and downloading the files described in the
+    catalog.
 
     This utility can be installed using `pip`:
 
     `$ python3 -m pip install springer`
 
-    Or the latest from master:
+    Or from the latest source on GitHub:
 
     `$ python3 -m pip install git+https://github.com/JnyJny/springer_downloader`
 
     The source is available on [GitHub](https://github.com/JnyJny/springer_downloader).
 
-    Catalogs are lists of books in specific _language_, spanning a _topic_. Catalogs
-    are further subdivided into _packages_ which are books grouped by subtopics.
+    Catalogs are lists of books in a specific _language_, spanning a _topic_. Catalogs
+    are further subdivided into _packages_ which are books grouped by sub-topics.
+    
+    Textbooks can be downloaded by; specific title, titles belonging
+    to eBook packages or all textbooks in a catalog. Titles and package names can be
+    incompletely specified and will download the best matches.
 
-    The available languages are:
-    \b
-    - English 
-    - German
+    The available languages are: English & German.
 
-    The available topics are:
+    The available topics are: _All Disciplines_ and _Emergency Nursing_.
 
-    \b
-    - `All Disciplies`, all, 
-    - `Emergency Nursing`, med.
-
-    Note: The Emergency Nursing topic is not currently available in English.
+    **Note: The _Emergency Nursing_ topic is not available in English.**
     """
 
     # EJO The callback function is called before any of the command functions
@@ -112,33 +116,37 @@ def main(
 def get_default_catalog_subcommand():
     """Print the default catalog identifier.
 
-    This is the default catalog that will be used when listing books and packages.
+    This is the default catalog that will be used when listing books and packages
+    and the user has not specified a --language or --topic on the command line.
     """
-    print(Catalog())
+
+    typer.secho(f"Default: {Catalog(fetch=False)}", fg="green")
 
 
 @cli.command(name="set-default-catalog")
-def set_default_catalog_subcommand(ctx: typer.Context):
+def set_default_catalog_subcommand(ctx: typer.Context,):
     """Set default catalog language and topic.
 
-    Examples
+    __Examples__
     
     Set the default catalog to German language:
 
-    `$ springer -L de set-default-catalog`
+    `$ springer --language de set-default-catalog`
 
     Set the default catalog to German and emergency nursing:
 
-    `$ springer -L de -T med set-default-catalog`
+    `$ springer --language de --topic med set-default-catalog`
 
     Set the default catalog to English and all disciplines topic:
 
-    `$ springer -L en -T all set-default-catalog`
+    `$ springer --language en --topic all set-default-catalog`
 
-    Note: The only English language catalog is en-all.
+    Note: The only English language catalog is `en-all`.
     """
+    prev = Catalog(fetch=False)
     ctx.obj.save_defaults()
-    get_default_catalog_subcommand()
+    typer.secho(f"Old Default: {prev!s}", fg="red" if prev.is_default else "blue")
+    typer.secho(f"New Default: {Catalog(fetch=False)!s}", fg="green")
 
 
 @cli.command(name="list")
@@ -157,11 +165,10 @@ def list_subcommand(
 ):
     """List books, package, packages, catalog or catalogs.
 
-    Display information about books, packages, and catalogs. Packages are
-    sets of books grouped by subject. There are currently three catalogs
-    available: en-all, de-all and de-med.
+    Display information about books, packages, and catalogs. Packages
+    are sets of books grouped by subject.
 
-    Examples
+    __Examples__
     
     List titles available in the default catalog:
 
@@ -190,7 +197,6 @@ def list_subcommand(
     List information about the Germal language, Emergency Nursing catalog:
 
     `$ springer --language de --topic med list catalog`
-
     """
 
     if component == Component.Books:
@@ -221,19 +227,19 @@ def list_subcommand(
         catalog.list_catalog(long_format)
 
 
-@cli.command(name="refresh")
+@cli.command(name="refresh-catalog")
 def refresh_subcommand(
     ctx: typer.Context,
     catalog_url: str = typer.Option(
-        None, "--url", "-u", help="URL for Excel-formatted catalog"
+        None, "--url", "-u", help="URL for Excel-formatted catalog."
     ),
     all_catalogs: bool = typer.Option(False, "--all", is_flag=True),
 ):
-    """Refresh the cached catalog of Springer textbooks.
+    """Refresh the cached catalog of springer textbooks.
 
-    If --all is specified, the --url option is ignored.
+    If `--all` is specified, the `--url` option is ignored.
 
-    Examples
+    __Examples__
 
     Update English language catalog:
 
@@ -243,13 +249,15 @@ def refresh_subcommand(
 
     `$ springer --language de --topic all refresh`
 
-    Update German language catalog whose topic is 'med' with a new URL:
+    Update German language catalog whose topic is 'med' with a new url:
 
-    `$ springer -L de -D med refresh --url https://example.com/api/endpoint/something/v11`
+    `$ springer -l de -d med refresh --url https://example.com/api/endpoint/something/v11`
+
+    __NOTE: THIS URL DOES NOT REPLACE THE DEFAULT URL FOR THE TARGET CATALOG__
 
     Update all catalogs:
 
-    `$ springer refresh --all`
+    `$ springer refresh-catalog --all`
 
     """
 
@@ -263,27 +271,27 @@ def refresh_subcommand(
         print(catalog)
 
 
-@cli.command(name="clean")
+@cli.command(name="clean-catalog")
 def clean_subcommand(
     ctx: typer.Context,
     force: bool = typer.Option(False, "--force", "-F", is_flag=True),
     all_catalogs: bool = typer.Option(False, "--all", is_flag=True),
 ):
-    """Removes the cached catalog.
+    """Remove cached catalogs.
 
-    Examples
+    __Examples__
 
     Remove the cached default catalog:
 
-    `$ springer clean --force`
+    `$ springer clean-catalog --force`
 
-    Remove the cached German language emergency nursing catalog:
+    Remove the cached German language _Emergency Nursing_ catalog:
 
-    `$ springer --language de --topic med clean --force`
+    `$ springer --language de --topic med clean-catalog --force`
 
     Remove all catalogs:
     
-    `$ springer clean --force --all`
+    `$ springer clean-catalog --force --all`
     """
 
     if not force:
@@ -301,6 +309,9 @@ def clean_subcommand(
 @cli.command(name="download")
 def download_subcommand(
     ctx: typer.Context,
+    title: str = typer.Option(
+        None, "--book-title", "-t", help="Book title to match (partial title OK)"
+    ),
     package: str = typer.Option(
         None, "--package-name", "-p", help="Package name to match (partial name OK)."
     ),
@@ -335,23 +346,22 @@ def download_subcommand(
 
     If a download is interrupted, you can re-start the download and it
     will skip over files that have been previously downloaded and pick up
-    where it left off. 
+    where it left off.
+
+    
 
     If the --all option is specified, the --dest-path option specifies the
     root directory where files will be stored. Each catalog will save 
     it's textbooks to:
     
-    dest_path/language/topic/book_file_name.fmt
+    `dest_path/language/topic/book_file_name.fmt`
 
-    Files that fail to download will be logged to a file named:
+    Errors encountered during the download will be logged to a file named:
 
-    dest_path/DOWNLOAD_ERRORS.txt
+    `dest_path/DOWNLOAD_REPORT.txt`
     
-    The log entries will have the date and time of the attempt,
-    the HTTP status code and the URL that was attempted.
 
-
-    EXAMPLES
+    __Examples__
 
     Download all books in PDF format to the current directory:
     
@@ -379,33 +389,76 @@ def download_subcommand(
 
     Download all books in the 'Computer Science' package in pdf format:
 
-    `$ springer download --package-name Computer`
+    `$ springer download --package-name computer`
     """
 
-    logger.configure(
-        **{
-            "handlers": [
-                {
-                    "sink": dest_path / DOWNLOAD_REPORT,
-                    "format": "{time:YYYY-MM-DD HH:mm} | <red>{message}</>",
-                    "colorize": True,
-                },
-            ]
-        }
-    )
+    # EJO Revamp invocation to match the list_subcommand ??
+    #
+    #     springer download catalog -d dest
+    #     springer download catalogs -d dest
+    #     springer download package -d dest -m package_re
+    #     springer download packages -d dest
+    #     springer download book -d dest -m book_re
+    #     springer download books -d dest
 
     dest_path = dest_path.resolve()
 
-    if not all_catalogs:
-        if package:
-            ctx.obj.download_package(package, dest_path, file_format, overwrite)
-            return
-        ctx.obj.download(dest_path, file_format, overwrite)
-        return
+    try:
+        logger.configure(
+            **{
+                "handlers": [
+                    {
+                        "sink": dest_path / DOWNLOAD_REPORT,
+                        "format": "{time:YYYY-MM-DD HH:mm} | <red>{message}</>",
+                        "colorize": True,
+                    },
+                ]
+            }
+        )
+        if not all_catalogs:
 
-    for catalog in Catalog.all_catalogs():
-        dest = dest_path / catalog.language.name / catalog.topic.value
-        if package:
-            catalog.download_package(package, dest_path, file_format, overwrite)
-            continue
-        catalog.download(dest, file_format, overwrite=overwrite)
+            dest_path.mkdir(mode=0o755, exist_ok=True, parents=True)
+
+            if title:
+                ctx.obj.download_title(title, dest_path, file_format, overwrite)
+                return
+
+            if package:
+                ctx.obj.download_package(package, dest_path, file_format, overwrite)
+                return
+
+            ctx.obj.download(dest_path, file_format, overwrite)
+            return
+
+        for catalog in Catalog.all_catalogs():
+            dest = dest_path / catalog.language.name / catalog.topic.value
+            dest.mkdir(mode=0o755, exist_ok=True, parents=True)
+            # EJO KeyError is caught in this loop since we are iterating on
+            #     all the catalogs and a failure in one catalog shouldn't stop
+            #     the show.
+            if title:
+                try:
+                    catalog.download_title(title, dest_path, file_format, overwrite)
+                except KeyError as error:
+                    typer.secho(f"{catalog}: ", nl=False)
+                    typer.secho(str(error).strip('"'), fg="red")
+                    continue
+
+            if package:
+                try:
+                    catalog.download_package(package, dest_path, file_format, overwrite)
+                except KeyError as error:
+                    typer.secho(f"{catalog}: ", nl=False)
+                    typer.secho(str(error).strip('"'), fg="red")
+                    continue
+
+            catalog.download(dest, file_format, overwrite=overwrite)
+
+    except KeyError as error:
+        typer.secho(str(error), fg="red")
+        raise typer.Exit(-1) from None
+
+    except PermissionError as error:
+        typer.secho("Permission error for: ", nl=False)
+        typer.secho(str(error.filename), fg="red")
+        raise typer.Exit(-1) from None
